/*
 * Copyright (c) 2018, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * WSO2 Inc. licenses this file to you under the Apache License,
 * Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

import Component from "./component";
import Instance from "./instance";
import InstanceList from "./InstanceList";
import NotFound from "../common/error/NotFound";
import React from "react";
import {Route, Switch, withRouter} from "react-router-dom";
import * as PropTypes from "prop-types";

const Instances = ({match}) => (
    <Switch>
        <Route exact path={`${match.path}/`} component={InstanceList}/>
        <Route exact path={`${match.path}/:instanceName`} component={Instance}/>
        <Route exact path={`${match.path}/:instanceName/components/:componentName`} component={Component}/>
        <Route path={`${match.url}/*`} render={(props) => <NotFound {...props} showNavigationButtons={true}/>}/>
    </Switch>
);

Instances.propTypes = {
    match: PropTypes.object.isRequired,
    location: PropTypes.shape({
        state: PropTypes.object
    }).isRequired
};

export default withRouter(Instances);
